/*
program: CEDIT.cmd
type:    REXXSAA-OS/2, version 2.x
purpose: Edit Config.SYS File
version: 1.0
date:    1992-08-27 (Internal)
changed: 1.1 1992-09-13 (Internal)
             -SET logic to allow additon of variable
              if it doesn't currently exist
             -Allow no duplicate directory adds on path or set statements
                (only valid if input has semicollons!!)
         2.0 1992-10-06 (External)
             -Can Now accept flags from the command line.
             -Commands changed for clarity.

author:  Byron Stephan,
         Email through OS/2 Shareware BBS

needs:   OS/2 2.0 & Rexx Fixes

Plans:    Need to standardize the Input logic.
          Should check when a directory is involved that a Semi-colon is on the end.
          Should have SET input flag as error attemp to modify PATH, DPATH, ...
          ?Error if SET line previously had a semi-colon ending it?

Problems: none known
          Email through OS/2 Shareware BBS if any detected please!

All rights reserved, copyrighted 1992, no guarantee that it works without
errors, etc. etc.

donated to the public domain granted that you are not charging anything (money
etc.) for it and derivates based upon it, as you did not write it,
etc. if that holds you may bundle it with commercial programs too

you may freely distribute this program, granted that no changes are made
to it.

Please, if you find an error, post me a message describing it, I will
try to fix and rerelease it to the net.

procedures:


usage:   CEDIT [/F:filename]
         CEDIT [/E:config.sys line]
         CEDIT [/B:config.sys line]
         CEDIT [/D:directories]
         CEDIT [/L:directories]
         CEDIT [/P:directories]
         CEDIT [/S:variable=value]

         CEDIT [/?] _____
         CEDIT [/H]      \ --- Help
         CEDIT      _____/

         CEDIT [/FH] Command File Help

syntax:
    FILENAME .... Filename of commands

flags:
    /B:  ........ Add at beginning of Config.sys
    /D:  ........ Add to DPATH variable
    /E:  ........ Add at end of config.sys
    /L:  ........ Add to LIBPATH Variable
    /P:  ........ Add to PATH Variable
    /S:  ........ Add to a SET Variable

*/

/****************************************************************
** Set up error handling.
** Intialize.
** Parse Command line arguments.
** Process commands.
*****************************************************************/


SIGNAL ON FAILURE NAME Error
SIGNAL ON HALT NAME Halt
SIGNAL ON ERROR NAME Error
SIGNAL ON SYNTAX NAME Error

global. = ""

CALL Initialize                 /* initialize array "global."           */
CALL parse_args ARG(1)
CALL Process
exit


Initialize: PROCEDURE EXPOSE global.
/****************************************************************
** Define colors.
** Load necessary functions if not already loaded.
** Initialize global variables.
*****************************************************************/

/* define some ANSI.SYS-colors */
esc    = '1B'x          /* escape-char */
    
/* define ANSI-colors */
global.iRedWhite  = esc||"[31;47m"    /* ANSI.SYS-control for red foreground */
global.iRed       = esc||"[31;40m"    /* ANSI.SYS-control for yellow foreground */
global.iGreen     = esc||"[32;40m"    /* ANSI.SYS-control for yellow foreground */
global.iYellow    = esc||"[33;40m"    /* ANSI.SYS-control for yellow foreground */
global.iBlue      = esc||"[34;40m"    /* ANSI.SYS-control for yellow foreground */
global.iMagenta   = esc||"[35;40m"    /* ANSI.SYS-control for yellow foreground */
global.iCyan      = esc||"[36;40m"    /* ANSI.SYS-control for cyan foreground */
global.iNormal    = esc||"[0;40m"     /* ANSI.SYS-control for resetting attributes to normal */
global.iScrLength = LENGTH(global.iRedWhite)

/* Set Colorful program prompt */
global.CEDIT = global.iYellow||"CEDIT"||global.iNormal

/* hint for options */
global.defaultHint = global.iCyan || "(" || global.iYellow || "default" || global.iCyan || ")"
/* hint for default yes or no */
global.yesHint = global.iCyan || " (" || global.iYellow || "Y" || global.iCyan || "/N) " || global.iYellow
global.noHint  = global.iCyan || " (Y/" || global.iYellow || "N" || global.iCyan || ") " || global.iYellow

/* check whether RxFuncs are loaded, if not, load them */
IF RxFuncQuery('SysLoadFuncs') THEN do
   /* load the load-function */
   CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'       

   /* load the Sys* utilities */
   CALL SysLoadFuncs                                                 
END

/* get screen dimensions */
PARSE VALUE SysTextScreenSize() WITH row columns
global.iRows = row
global.iColumns = columns
global.iBlankLine = global.iRedWhite || RIGHT("", columns - 1) || global.iCyan

global.cLibpath = ""
global.cPath    = ""
global.cDpath   = ""
global.AddBegin = ""
global.AddEnd  = ""
global.AddBegin.0 = 0
global.AddEnd.0  = 0
global.SetVar.0  = 0

RETURN

Parse_ARGS: PROCEDURE EXPOSE Global.
/****************************************************************
** Check for no arguments or Help switches.
** Parse various switches until no more switches.
** --While switches exist...
** --   Find next switch
** --   Parse Switch & Value to seperate them out
** --   (Note: Requires that a colon exist as part of the switch)
** --   For switch do appropriate action
*****************************************************************/

IF ARG(1) = '' | POS('/?',ARG(1)) | POS('/H',TRANSLATE(ARG(1))) THEN
   SIGNAL Usage
IF POS('/FH',TRANSLATE(ARG(1))) THEN
   SIGNAL FileUsage


SwitchPos = pos('/',ARG(1))
do while SwitchPos > 0
   EndSwitchPos = pos('/',value,SwitchPos+3) - 1
   if EndSwitchPos < 1 then
      EndSwitchPos = Length(arg(1))

   Switch_value = substr(arg(1),SwitchPos,EndSwitchPos)

   parse value Switch_Value with '/' Switch ':' Value 
   Switch = TRANSLATE(Switch)
   Select
      when Switch = 'B' then do
         global.AddBegin.0 = global.AddBegin.0 + 1
         iTemp = global.AddBegin.0
         global.AddBegin.iTemp = value
      end  /* Do */
      when Switch = 'E' then do
         global.AddEnd.0 = global.AddEnd.0 + 1
         iTemp = global.AddEnd.0
         global.AddEnd.iTemp = value
      end  /* Do */
      when Switch = 'F' then do
         global.filein = STRIP(Value)          /* get rid of leading and trailing spaces */
         global.eFilein = STREAM(global.filein, "C", "QUERY EXISTS") /* File exists? */

         IF global.eFilein = "" THEN do
            SAY global.CEDIT": file "||global.iGreen||"["||global.iCyan || global.filein || global.iGreen||"]"||global.iNormal" does not exist."
            EXIT -2
         END
         else
            call Read_File
      end  /* Do */
      when Switch = 'L' then
         global.cLibpath = global.cLibpath||value
      when Switch = 'D' then
         global.cDpath = global.cDpath||value
      when Switch = 'P' then
         global.cPath = global.cPath||value
      when Switch = 'S' then do
         /* If SET keyword exists then add value to keywords value 
         ** else add new SET keyword and value. 
         */ 
         eVariableFound = 0
         PARSE VALUE value WITH keyword "=" value

         DO iTemp = 1 TO global.setvar.0
            if global.setvar.Keys.itemp = TRANSLATE(keyword) then do
               global.SetVar.KeyValue.iTemp = global.SetVar.KeyValue.iTemp||value
               eVariableFound = 1
            END
         END
         if eVariableFound = 0 then DO
            global.SetVar.0 = global.SetVar.0 + 1
            iTemp = global.SetVar.0

            global.SetVar.Keys.iTemp = TRANSLATE(keyword)
            global.SetVar.KeyValue.iTemp   = value
            global.SetVar.SetExists.iTemp = 0
         end
      END

   otherwise
      /* Unknown switch - show help */
      Signal Usage

end  /* select */

RETURN

Read_File: PROCEDURE EXPOSE global.
/****************************************************************
** While not EOF
**    Read line. Ignore comments and blank lines.
**    Parse file input
**    if problem show line and exit
*****************************************************************/
SAY "Reading Command file..."
iline_no = 0             /* line-number */
DO WHILE LINES(global.filein) > 0
   line = LINEIN(global.filein)
   iline_no = iline_no + 1

   /* no empty lines and no comments */
   IF line = "" | SUBSTR(STRIP(line), 1, 2) = "--" THEN ITERATE

   SAY "parsing "||global.iGreen||"["||global.iNormal || line || global.iGreen||"]"||global.iNormal

   IF Parse_File_Input(line, iline_no) > 0 THEN do
      SAY global.iredwhite
      SAY "Line No:" iline_no
      SAY ">>> " line global.inormal
      SAY 
      exit -3
   END
END

CALL STREAM global.filein, "C", "CLOSE"        /* make sure file is closed */

RETURN

Process: PROCEDURE EXPOSE global.
/****************************************************************
** Find Boot drive
** Update temporary file.
** Backup Config.sys file
** If no problems then replace config.sys with temporary file.
*****************************************************************/

global.cBootdrive = substr(VALUE('PATH',,'OS2ENVIRONMENT'),POS('\OS2\SYSTEM',VALUE('PATH',,'OS2ENVIRONMENT')) - 2,2)

SAY "Creating temporary file..."

cTempFilename = SysTempFileName(Global.cBootdrive||'\Temp????.SYS')
DO WHILE LINES(global.cbootdrive||"\CONFIG.SYS") > 0
   line = LINEIN(global.cbootdrive||"\CONFIG.SYS")
   UPPERline = TRANSLATE(LINE)

   SELECT
      WHEN ("IFS=" = SUBSTR(UPPERline,1,4)) THEN
         DO
         CALL LINEOUT cTempFilename, line
         DO iTemp = 1 TO global.AddBegin.0
            CALL LINEOUT cTempFilename,Global.AddBegin.iTemp
         END
         END
      WHEN ("LIBPATH=" = SUBSTR(UPPERline,1,8)) THEN
         do
         Path = CheckPath(upperline,Global.cLibpath)
         CALL LINEOUT cTempFilename,path 
         end
      WHEN ("SET PATH=" = SUBSTR(UPPERline,1,9)) THEN
         do
         Path = CheckPath(upperline,Global.cPath)
         CALL LINEOUT cTempFilename,Path 
         end
      WHEN ("SET DPATH=" = SUBSTR(UPPERline,1,10)) THEN
         do
         Path = CheckPath(upperline,Global.cDpath)
         CALL LINEOUT cTempFilename,path 
         end
      WHEN ("SET " = SUBSTR(UPPERline,1,4)) THEN
         do
         if global.setvar.0 = 0 then
            CALL LINEOUT cTempFilename,line 
         else
            DO
            eKeywordFound = 0
            DO iTemp = 1 TO global.SetVar.0
               PARSE VAR UPPERline with "SET " keyword "=" 
               if global.setvar.keys.iTemp = keyword then
                  DO
                  Path = CheckPath(upperline,Global.SetVar.KeyValue.iTemp,"Set")
                  CALL LINEOUT cTempFilename,path
                  eKeywordFound = 1
                  global.SetVar.SetExists.iTemp = 1
                  END
            END
            if eKeywordFound = 0 then
               CALL LINEOUT cTempFilename,line 
            END
         END
    OTHERWISE
         CALL LINEOUT cTempFilename,line 
   END
END

DO iTemp = 1 TO global.SetVar.0
   if global.setvar.SetExists.iTemp = 0 then do
       Path = CheckPath("",Global.SetVar.KeyValue.iTemp,"Set")
       CALL LINEOUT cTempFilename, "SET "||global.setvar.keys.iTemp||"="||path
   END
END


DO iTemp = 1 TO global.AddEnd.0
   CALL LINEOUT cTempFilename,Global.AddEnd.iTemp 
END
CALL STREAM cTempFilename, "C", "CLOSE"        /* make sure file is closed */
CALL STREAM global.cbootdrive||"\CONFIG.SYS", "C", "CLOSE"        /* make sure file is closed */

call backup_file global.cbootdrive||"\CONFIG.SYS"

SAY "Updating Config.SYS file..."

"@copy "||cTempFilename||" "||global.cbootdrive||"\CONFIG.SYS"
RC = SysFileDelete(cTempFilename)
if RC = 0 then
   exit 0
else
   do
   say global.red||"ERROR!! File could not be deleted!!"||global.inormal
   exit -2
   end
*/
Say "Finished..."
return 0

Backup_File: PROCEDURE EXPOSE global.
/****************************************************************
** Backup file with Backini if exists
** else do generic backup
*****************************************************************/
original_file = arg(1)

SAY "Backing up Config.SYS file..."

BackiniPath = SysSearchPath('PATH','Backini.exe')
if BackiniPath<>"" then
   "@"||BackiniPath||" "||original_file||" 25 "
else do
   iBackup = 0
   DO FOREVER
      iBackup = iBackup + 1
      extension = lastpos(".", original_file)
      if extension = 0 then do
         extension = length(original_file) + 1
         backup_file = original_file||"."||iBackup
      end
      else
         backup_file = substr(original_file,1,extension)||iBackup

      global.eBackup = STREAM("CONFIG."||iBackup, "C", "QUERY EXISTS")
 
      IF global.eBackup = "" THEN do
         "@copy "||original_file||" "||backup_file
         leave
      END /* End-If */
   END /* End-DO */
END /* End-Else */

return 0

Parse_File_Input: PROCEDURE EXPOSE global.
/****************************************************************
** Parse file records for keywords.
** If unknown keyword exists then return bad error code.
*****************************************************************/

to_parse = ARG(1)
line_no  = ARG(2)
iErrorExists = 0

PARSE VALUE to_parse WITH keyword "=" value

SELECT
   WHEN translate(keyword) = "LIBPATH" THEN
      global.cLibpath = global.cLibpath||value
   WHEN translate(keyword) = "DPATH" THEN
      global.cDpath = global.cDpath||value
   WHEN translate(keyword) = "PATH" THEN
      global.cPath = global.cPath||value
   WHEN translate(keyword) = "ADDBEGIN" THEN
      DO
      global.AddBegin.0 = global.AddBegin.0 + 1
      iTemp = global.AddBegin.0
      global.AddBegin.iTemp = value
      END
   WHEN translate(keyword) = "ADDEND" THEN
      DO
      global.AddEnd.0 = global.AddEnd.0 + 1
      iTemp = global.AddEnd.0
      global.AddEnd.iTemp = value
      END
   WHEN translate(keyword) = "SETVAR" THEN
      DO
      eVariableFound = 0
      PARSE VALUE value WITH keyword "=" value

      DO iTemp = 1 TO global.setvar.0
         if global.setvar.Keys.itemp = TRANSLATE(keyword) then
            DO
            global.SetVar.KeyValue.iTemp = global.SetVar.KeyValue.iTemp||value
            eVariableFound = 1
            END
      END         
      if eVariableFound = 0 then
         DO
         global.SetVar.0 = global.SetVar.0 + 1
         iTemp = global.SetVar.0 

         global.SetVar.Keys.iTemp = TRANSLATE(keyword)
         global.SetVar.KeyValue.iTemp   = value
         global.SetVar.SetExists.iTemp = 0
         end
      END
   OTHERWISE
      iErrorExists = iErrorExists + 1
END

RETURN iErrorExists

CheckPath: PROCEDURE
/****************************************************************
** If new value for SET statment doesn't have semi-colon 
**    Replace SET value
** else 
**    Check PATH being built for duplicate directory.
*****************************************************************/
line = arg(1)
value = arg(2)
Source = arg(3)
path = line
startingpos = pos('=',value) + 1
lastpos = pos(';',value)
if translate(source) = 'SET' & lastpos = 0 then do
   SetStatement = substr(line,1,pos('=',line))||value
   return SetStatement
end
else
   do while lastpos > 0
      subvalue = translate(substr(value,startingpos,lastpos-startingpos+1))
      if pos(subvalue, path) > 0 then
         nop /*Say 'Value '||subvalue||' already exists in '||line*/
      else
         path = path||subvalue
      Startingpos = lastpos + 1
      lastpos = pos(';',value,startingpos)
   end
return path

USAGE:
/****************************************************************
** Help on switches
*****************************************************************/
   
   SAY global.cedit||": Modifies Config.SYS file"
   SAY
   SAY "      /"||global.iMagenta||"H"||global.iNormal"                  ... This Screen of switches"||global.iNormal
   SAY "      /"||global.iMagenta||"FH"||global.iNormal"                 ... Help for File commands"||global.iNormal
   SAY "      /"||global.iMagenta||"F:"||global.iGreen||"["||global.iCyan||"filename"||global.iGreen||"]"||global.iNormal"       ... contains file of commands"||global.iNormal
   SAY "      /"||global.iMagenta||"L:"||global.iGreen||"["global.icyan||"directory;"||global.iGreen||"]"||global.iNormal
   SAY "      /"||global.iMagenta||"P:"||global.iGreen||"["global.icyan||"directory;"||global.iGreen||"]"||global.iNormal
   SAY "      /"||global.iMagenta||"D:"||global.iGreen||"["global.icyan||"directory;"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......         "||global.ired||"(Must use ; after directory names!) "||global.inormal
   SAY "      /"||global.iMagenta||"B:"||global.iGreen||"["global.icyan||"Config.SYS line"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......         "||global.icyan||"(After File System is loaded... After IFS Statement) "||global.inormal
   SAY "      /"||global.iMagenta||"E:"||global.iGreen||"["global.icyan||"Config.SYS line"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......         "||global.icyan||"(Add to end of Config.SYS file) "||global.inormal
   SAY "      /"||global.iMagenta||"S:"||global.iGreen||"["global.icyan||"Keyword"||global.iGreen||"]"||global.iNormal||"="||global.iGreen||"["global.icyan||"Value"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......     "||global.iyellow||"Keyword"||global.inormal||":"||global.icyan||" Keyword for SET Statment "||global.inormal
   SAY global.iBlue||"......     "||global.iyellow||"Value"||global.inormal||":"||global.icyan||" Value to SET keyword "||global.inormal
   SAY global.iBlue||"......   "||global.ired||"(Must use ; if VALUE is directory names!) "||global.inormal
   SAY global.iBlue||"......   "||global.ired||"(Set Variable cannot have ""/"" for it's value - Do through command file) "||global.inormal

   SAY

   EXIT -1

FileUsage:
/****************************************************************
** Help on File Commands.
*****************************************************************/
   
   SAY global.cedit||": Modifies Config.SYS file"
   SAY "    : File Format"
   SAY 
   SAY "            "||global.iMagenta||"--"||global.iNormal||"                  ... Comment Line"
   SAY "       "||global.iMagenta||"LIBPATH"||global.inormal||"="||global.iGreen||"["global.icyan||"directory;"||global.iGreen||"]"||global.iNormal
   SAY "          "||global.iMagenta||"PATH"||global.inormal||"="||global.iGreen||"["global.icyan||"directory;"||global.iGreen||"]"||global.iNormal
   SAY "         "||global.iMagenta||"DPATH"||global.inormal||"="||global.iGreen||"["global.icyan||"directory;"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......         "||global.ired||"(Must use ; after directory names!) "||global.inormal
   SAY "      "||global.iMagenta||"AddBegin"||global.inormal||"="||global.iGreen||"["global.icyan||"Config.SYS line"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......         "||global.icyan||"(After File System is loaded... After IFS Statement) "||global.inormal
   SAY "        "||global.iMagenta||"AddEnd"||global.inormal||"="||global.iGreen||"["global.icyan||"Config.SYS line"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......         "||global.icyan||"(Add to end of Config.SYS file) "||global.inormal
   SAY "        "||global.iMagenta||"SetVar"||global.inormal||"="||global.iGreen||"["global.icyan||"Keyword"||global.iGreen||"]"||global.iNormal||"="||global.iGreen||"["global.icyan||"Value"||global.iGreen||"]"||global.iNormal
   SAY global.iBlue||"......         "||global.iyellow||"Keyword"||global.inormal||":"||global.icyan||" Keyword for SET Statment "||global.inormal
   SAY global.iBlue||"......         "||global.iyellow||"Value"||global.inormal||":"||global.icyan||" Value to SET keyword "||global.inormal
   SAY global.iBlue||"......         "||global.ired||"(Must use ; if VALUE is directory names!) "||global.inormal

   SAY

   EXIT -1

HALT:
/****************************************************************
** User stopped program! Shame on them!!
*****************************************************************/
   CALL STREAM filein, "C", "CLOSE"             /* make sure, file is closed */
   SAY global.cedit||": User interrupted program."
   EXIT -1

ERROR:        
/****************************************************************
** Error of some sort occured!
*****************************************************************/
   myrc = RC
   SAY global.cedit||': error occurred !'
   SAY

   SAY 'REXX error' myrc 'in line' SIGL':' ||global.iRedWhite||ERRORTEXT(myrc)||global.iNormal
   SAY global.iRed||Substr('     ',1,6-Length(SIGL))(SIGL)' *-*   'Sourceline(sigl)||global.iNormal

IF MYRC = 43 THEN
   SAY 'Chances are you are in OS/2 1.3 where this routine will not work.'
   SAY
   SAY global.iCyan||'Please contact the author with a description of the error.'||global.iNormal

   EXIT -99

