/*   A REXX program to calculate gears for specific size wheels.


    FileName:  GEARS.CMD
    Date:      February 24, 1994
    Author:    Chris Hascall
               49 McKinley Street
               South Portland  ME  04106
               USA   (207) 799-8679
    Purpose:   To calculate gears for bicycle wheel siz(es)

    -----------------------------------------------
*/

/* trace all */


main:

   call PullWheels

   call rxfuncadd 'rxydbautilinit','ydbautil','rxydbautilinit'
   call rxydbautilinit

   call SysCls

   RxGearsFile = 'd:\temp\unzip\ringgear.dat'
   RxSprockFile = 'd:\temp\unzip\sprocket.dat'
   RxOutFile = 'd:\temp\unzip\gears.file.' || FileExt
   WheelSize = WhDiam
   Sprocket = 0

   pos = SysCurPos(1, 13)
   say 'Calculation Results for your Selections'

   call readGearsFileS
   call calculateGear
   call writeFileHeaders
   call writeCalculations

   exit

/* ------------------------------------------------- */
PullWheels:

   call SysCls

   Do until datatype(WhDiam) = 'NUM'
     say 'Type the diameter of wheel to calculate:'
     say 'This should be in inches - ie: "26.66"'
     say ' '
     pull WhDiam
     if datatype(WhDiam) = 'CHAR' then
     do
        say 'Not a number. Try again!'
     end
   end

   len = length(WhDiam)
   FileExt = substr(WhDiam, 1, len)
   WhDiam = format(WhDiam, 4, 2)

   call SysCls
   say ' '
   say 'Select options:'
   say '  '
   say 'Press "C" - Chart only'
   say 'Press "D" - Chart and specific gears'

   key = SysGetKey('NOECHO')

   if key = 'D' | key = 'd' then
   do
      SelectedKey = 'D'
      call PullGears
   end
   else
   do
      SelectedKey = 'C'
   end

   return

/* ------------------------------------------------- */
PullGears:

   call SysCls
   say 'Type each ChainWheel size to be calculated. After'
   say 'the last has been entered, type a spaceline!'
   say 'Please type whole numbers!'
   say ' '

   n = 1
   do forever
      say 'Type size for ChainWheel # 'n
      pull ChainW.n
      select
         when datatype(ChainW.n, 'W') then
            n = (n + 1)
         when ChainW.n = ' ' then
            leave
         otherwise
            say 'ChainWheel must be a whole number'
      end
   end
   n = (n - 1)

   call SysCls

   return

/* ------------------------------------------------- */
readGearsFileS:

   info = rxexeci(RxGearsFile,,'chainring.','s')

   info2 = rxexeci(RxSprockFile,,'Sprock.','s')

   return

/* ------------------------------------------------- */
writeFileHeaders:

   SprocketLine = '   '
   DashLine = ' '

   do jj = 12 to 34
      SprocketLine = SprocketLine || '     ' || jj
   end
   len = length(SprocketLine)

   do kk = 1 to len
      DashLine = DashLine || '-'
   end

   BuildText = 'Sprocket Sizes & Gear Calculations - (' FileExt 'Inch Diameter wheel)'
   SprocketText = center(BuildText, 90, ' ')
   SprocketText = 'Size' || substr(SprocketText, 6, 85)

   call lineout RxOutFile, DashLine
   call lineout RxOutFile, 'Chain Ring'
   call lineout RxOutFile, SprocketText
   call lineout RxOutFile, SprocketLine
   call lineout RxOutFile, DashLine
   call lineout RxOutFile

   return

/* ------------------------------------------------- */
calculateGear:

   ChainRingLine = '   '

   say ' '
   say ' '

   row = 5
   hrow = 4
   col = 10
   tt = 0
   qq = 0
   z = n

   do ii = 1 to chainring.0
      GearLine.ii = ' ' || ChainRing.ii
      ChainSize = datatype(ChainRing.ii,'N')
      if ChainSize = 1 then
      do
         if z > -1 then
         do
            doc = ProcessInput(0)
         end
         else
         do
            doc = 0
         end

         do Sprocket = 10 to 34
            Numerator = (WhDiam * ChainRing.ii)
            GearSize = format((Numerator / Sprocket), 4, 1)
            gearline.ii = GearLine.ii || ' ' || GearSize
            if doc = 1 then
            do
               dix = ProcessSprock(0)
               if dix = 1 then
               do
                  pos = SysCurPos(hrow, (col + 3))
                  say Sprocket
                  pos = SysCurPos(row, col)
                  say GearSize
                  col = col + 8
               end
            end
         end
         col = 10
      end
   end

   return

/* ------------------------------------------------- */
ProcessInput:

Arg retc

   do tt = 1 to n

      intype = datatype(ChainW.tt, 'N')

      if intype = 1 then
      do
        if (ChainW.tt = ChainRing.ii) then
        do
           z = z - 1
           row = row + 1
           pos = SysCurPos(row, (col - 5))
           say ChainRing.ii
           return intype
        end
      end
   end

   return 0

/* ------------------------------------------------- */
ProcessSprock:

Arg retd

   do qq = 1 to Sprock.0

      if (Sprock.qq = Sprocket) then
      do
         return 1
      end
   end

   return 0

/* ------------------------------------------------- */
writeCalculations:

   info = rxexeco(RxOutFile,'a', GearLine.,'s')

   return
